/*
   SoundPref
   $Revision: 0.1 $
   Tatsuya Aoyagi (aoyagi@cs.uec.ac.jp)
   $Id: sndpref.c,v 0.1 1997/08/06 11:35:44 aoyagi Exp aoyagi $
*/
#pragma pack(2)

#include <Pilot.h>
#include "sndpref.h"
#include "prefdb.h"
#include "play.h"

#include "sndpref.rid"

DmOpenRef PrefDB;
char** Choice;
MelodyRecord* Melodies;
UInt NumMelodies;

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {
  int error;
  if (cmd == sysAppLaunchCmdNormalLaunch) {
    error = StartApplication();
    if (error) return error;
    EventLoop();
    StopApplication();
  }
  return 0;
}

int StartApplication(void) {
  PrefDB = openPrefDatabase();
  if (PrefDB == 0) { return 1; }
  NumMelodies = countMelody();
  NumMelodies += 2; // for default entry.
  Choice = (char**)MemPtrNew(sizeof(char*)*NumMelodies);
  if (Choice == 0) { return 1; }
  Melodies = (MelodyRecord*)MemPtrNew(sizeof(MelodyRecord)*NumMelodies);
  if (Melodies == 0) {
    MemPtrFree(Choice);
    return 1;
  }
  FrmGotoForm(MainFormID);
  return 0;
}

void StopApplication(void) {
  MemPtrFree(Choice);
  MemPtrFree(Melodies);
  DmCloseDatabase(PrefDB);
}

void EventLoop(void) {
  short err;
  EventType e;
  Boolean handled;

  while(1) {
    EvtGetEvent(&e,-1);
    if (SysHandleEvent(&e)) continue;
    if (MenuHandleEvent((void *)0, &e, &err)) continue;

    handled = false;
    switch (e.eType) {
    case appStopEvent:
      return;
    case frmLoadEvent:
      FrmSetActiveForm(FrmInitForm(e.data.frmLoad.formID));
      handled = true;
      break;
    case frmOpenEvent:
      setupList(PrefDB,Choice,Melodies,NumMelodies);
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;
    case menuEvent:
      FrmAlert(AboutAlertID);
      handled = true;
      break;
    case popSelectEvent:
      selectMelody(PrefDB,Melodies,e.data.popSelect.listID,e.data.popSelect.selection);
      handled = false;
      break;
    case ctlSelectEvent:
      switch (e.data.ctlSelect.controlID) {
      case SetAllDefaultButtonID:
	setAllList(PrefDB,Choice,Melodies,DefaultSoundIndex);
	handled = true;
	break;
      case SetAllNoneButtonID:
	setAllList(PrefDB,Choice,Melodies,NoneSoundIndex);
	handled = true;
	break;
      case InfoButtonID: case WarningButtonID: case ErrorButtonID: case StartUpButtonID:
      case AlarmButtonID: case ConfirmButtonID: case ClickButtonID:
	playSystemSound(PrefDB,ButtonIDToBeepID(e.data.ctlSelect.controlID));
	handled = true;
	break;
      }
      break;
    }
    if (!handled) {
      FrmHandleEvent(FrmGetActiveForm(), &e);
    }
  }
}

void setAllList(DmOpenRef prefdb,char** choice,MelodyRecord* mr,Word itemNo) {
  FormPtr form;
  UInt beepID;
  form = FrmGetActiveForm();
  for (beepID = sndInfo; beepID <= sndClick; beepID++) {
    setSoundPref(prefdb,beepID,&(mr[itemNo].sound));
    setLabelToPopupTrigger(form,BeepIDToPopupID(beepID),choice,itemNo);
  }
}

void playSystemSound(DmOpenRef prefdb,UInt beepID) {
  HackedSystemSound sound;

  getSoundPref(prefdb,beepID,&sound);
  switch (sound.kind) {
  case systemSoundMelody:
    playMelody(sound.melody.creator,sound.melody.resourceID);
    break;
  case systemSoundDefault:
    SndPlaySystemSound(beepID);
    break;
  }
}

void selectMelody(DmOpenRef prefdb,MelodyRecord* mr,Word listID,Word select) {
  setSoundPref(prefdb,ListIDToBeepID(listID),&(mr[select].sound));
}

void setupList(DmOpenRef prefdb,char** choice,MelodyRecord* mr,UInt n) {
  setDefaultEntry(mr);
  fillMelodyRecord(mr+2);
  fillChoice(choice,mr,n);
  setChoiceToAllList(prefdb,choice,mr,n);
}

void setDefaultEntry(MelodyRecord* mr) {
  setMelodyRecord(mr+DefaultSoundIndex,"default",systemSoundDefault,0,0);
  setMelodyRecord(mr+NoneSoundIndex,"none",systemSoundNone,0,0);
/*
  StrCopy(mr[DefaultSoundIndex].title,"default");
  mr[DefaultSoundIndex].sound.kind = systemSoundDefault;
  mr[DefaultSoundIndex].sound.melody.creator = 0;
  mr[DefaultSoundIndex].sound.melody.resourceID = 0;
  StrCopy(mr[NoneSoundIndex].title,"none");
  mr[NoneSoundIndex].sound.kind = systemSoundNone;
  mr[NoneSoundIndex].sound.melody.creator = 0;
  mr[NoneSoundIndex].sound.melody.resourceID = 0;
*/
}

void setChoiceToAllList(DmOpenRef prefdb,char** choice,MelodyRecord* mr,UInt n) {
  FormPtr form;
  UInt beepID;
  Word selectItemNo;
  form = FrmGetActiveForm();
  for (beepID = sndInfo; beepID <= sndClick; beepID++) {
    selectItemNo = currentSelection(prefdb,beepID,mr,n);
    setChoiceToList(form,BeepIDToListID(beepID),choice,n,selectItemNo);
    setLabelToPopupTrigger(form,BeepIDToPopupID(beepID),choice,selectItemNo);
  }
}

Word currentSelection(DmOpenRef prefdb,UInt beepID,MelodyRecord* mr,UInt n) {
  HackedSystemSound sound;
  UInt i;

  getSoundPref(prefdb,beepID,&sound);
  if (sound.kind == systemSoundDefault) { return DefaultSoundIndex; }
  if (sound.kind == systemSoundNone) { return NoneSoundIndex; }
  for (i = 2; i < n; i++) {
    if (sound.melody.creator == mr[i].sound.melody.creator &&
	sound.melody.resourceID == mr[i].sound.melody.resourceID) {
      return i;
    }
  }
  return 0;
}

void setChoiceToList(FormPtr form,Word objID,char** choice,UInt n,Word itemNum) {
  Word index;
  ListPtr list;
  index = FrmGetObjectIndex(form,objID);
  list = (ListPtr)FrmGetObjectPtr(form,index);
  LstSetListChoices(list,choice,n);
  LstSetSelection(list,itemNum);
}

void setLabelToPopupTrigger(FormPtr form,Word objID,char** choice,Word itemNo) {
  Word index;
  ControlPtr ctr;
  index = FrmGetObjectIndex(form,objID);
  ctr = (ControlPtr)FrmGetObjectPtr(form,index);
  CtlSetLabel(ctr,choice[itemNo]);
}

UInt countMelody(void) {
  UInt n;
  Boolean newSearch;
  DmSearchStateType state;
  ULong type,creator;
  Boolean onlyLatestVers;
  UInt cardNo;
  LocalID localID;

  newSearch = true;
  type = MelodyDBType;
  creator = 0; // wildcard
  onlyLatestVers = true;
  for (n = 0;
       DmGetNextDatabaseByTypeCreator(newSearch,&state,type,creator,
				      onlyLatestVers,&cardNo,&localID) == 0;
       newSearch = false) {
    n += countMelodyInDatabase(cardNo,localID);
  }
  return n;
}

UInt countMelodyInDatabase(UInt cardNo, LocalID dbID) {
  DmOpenRef dbref;
  UInt n;
  Int resID;
  dbref = DmOpenDatabase(cardNo,dbID,dmModeReadOnly);
  if (dbref == 0) { return 0; }
  for (resID = 1000, n = 0; DmFindResource(dbref,MelodyResourceType,resID,0) >= 0; resID++, n++) {
    ;
  }
  DmCloseDatabase(dbref);
  return n;
}

void fillMelodyRecord(MelodyRecord mr[]) {
  UInt n;
  Boolean newSearch;
  DmSearchStateType state;
  ULong type,creator;
  Boolean onlyLatestVers;
  UInt cardNo;
  LocalID localID;

  newSearch = true;
  type = MelodyDBType;
  creator = 0; // wildcard
  onlyLatestVers = true;
  for (n = 0;
       DmGetNextDatabaseByTypeCreator(newSearch,&state,type,creator,
				      onlyLatestVers,&cardNo,&localID) == 0;
       newSearch = false) {
    n += fillMelodyRecordInDatabase(cardNo,localID,creator,mr+n);
  }
}

UInt fillMelodyRecordInDatabase(UInt cardNo, LocalID dbID, ULong creator, MelodyRecord mr[]) {
  DmOpenRef dbref;
  UInt n;
  Int index;
  Int resID;
  VoidHand melhand;
  Melody* melody;
  UInt i;

  dbref = DmOpenDatabase(cardNo,dbID,dmModeReadOnly);
  if (dbref == 0) { return 0; }
  for (resID = 1000, n = 0; (index = DmFindResource(dbref,MelodyResourceType,resID,0)) >= 0; resID++, n++) {
/*
    mr[n].sound.kind = systemSoundMelody;
    mr[n].sound.melody.creator = creator;
    mr[n].sound.melody.resourceID = resID;
    melhand = DmGetResourceIndex(dbref,index);
    melody = (Melody*)MemHandleLock(melhand);
    for (i = 0; i < 32; i++) {
      mr[n].title[i] = melody->title[i];
    }
*/
    melhand = DmGetResourceIndex(dbref,index);
    melody = (Melody*)MemHandleLock(melhand);
    setMelodyRecord(mr+n,melody->title,systemSoundMelody,creator,resID);
    MemHandleUnlock(melhand);
    DmReleaseResource(melhand);
  }
  DmCloseDatabase(dbref);
  return n;
}

void fillChoice(char* choice[],MelodyRecord mr[],UInt n) {
  UInt i;
  for (i = 0; i < n; i++) {
    choice[i] = mr[i].title;
  }
}

void setMelodyRecord(MelodyRecord *mr,char* title,SystemSoundKind kind,ULong creator,Int resID) {
  UInt i;
  mr->sound.kind = kind;
  mr->sound.melody.creator = creator;
  mr->sound.melody.resourceID = resID;
  for (i = 0; i < sizeof(mr->title); i++) {
    mr->title[i] = title[i];
  }
}
