#! /usr/local/bin/perl
#  text to melody resource converter.
#  $Revision: 0.1 $
#  Tatsuya Aoyagi (aoyagi@cs.uec.ac.jp)
#  $Id: mel2bin.perl,v 0.1 1997/08/06 11:34:34 aoyagi Exp aoyagi $
#
#  useage: mel2bin.perl melodyfile ...

%NoteInterval = (
  'a'=>9,
  'b'=>11,
  'c'=>0,
  'd'=>2,
  'e'=>4,
  'f'=>5,
  'g'=>7,
);

$ResourceID = 1000;

setDefaults();

while (<>) {
  s/^\s+//;
  if (/^#/) { next; }
  if (/^title\s+"(([^"]|\\")*)"/i) {
    &flush_melody();
    setDefaults();
    $Title = $1;
  } elsif (/^pitch\s+(\d+)/i) {
    $Pitch = $1;
  } elsif (/^unittime\s+(\d+)/i) {
    $DefaultUnitTime = $1;
  } elsif (/^octave\s+(\d+)/i) {
    $DefaultOctave = $1;
  } elsif (/^volume\s+(\d+)/i) {
    $DefaultVolume = $1;
  } elsif (/^length\s+(\d+)/i) {
    $DefaultLenght = $1;
  } elsif (/^([a-g][#b]?)([-+]?\d+)?(\s+(\d+)(\s+(\d+))?)?/i) {
    $notename = $1;
    $octaveSpec = $2;
    $lengthSpec = $4;
    $volumeSpec = $6;
    $octave = octave($octaveSpec);
    $freq = freq($notename,$octave);
    $length = timeinmsec($lengthSpec);
    $volume = volume($volumeSpec);
    push(@Melody,$freq,$length,$volume);
#    print "$freq,$length,$volume\n";
  } elsif (/^(\d+)(\s+(\d+)(\s+(\d+))?)?/i) {
    $freq = $1;
    $lengthSpec = $3;
    $volumeSpec = $5;
    $length = timeinmsec($lengthSpec);
    $volume = volume($volumeSpec);
    push(@Melody,$freq,$length,$volume);
#    print "$freq,$length,$volume\n";
  } elsif (/^r(\s+(\d+))?/i) {
    $lengthSpec = $1;
    $length = timeinmsec($lengthSpec);
    $freq = 440;
    $volume = 0;
    push(@Melody,$freq,$length,$volume);
  }
}
flush_melody();

sub flush_melody {
  my ($freq,$msec,$vol);
  my ($filename,$i,$len);
  if (@Melody && $Title ne '') {
    $filename = "Mldy".sprintf("%04x",$ResourceID).".bin";
    $ResourceID++;
    if (open(F,">$filename")) {
      binmode(F);
      print "$Title\n";
      print F pack("a32",$Title);
      $len = @Melody / 3;
      print "$len\n";
      print F pack("n",$len);
      for ($i = 0; $i < $len; $i++) {
	$freq = shift(@Melody);
	$msec = shift(@Melody);
	$vol = shift(@Melody);
	print "$freq,$msec,$vol\n";
	print F pack("Nnn",$freq,$msec,$vol);
      }
      close(F);
    }
  }
}

sub freq {
  my ($notename,$octave) = @_;
  my ($note,$i,$sf);

  $notename =~ /^([a-g])([#b])?/i;
  $note = $1;
  $sf = $2;			  
  $note =~ y/A-Z/a-z/;
  $i = $NoteInterval{$note};
  if ($sf eq '#') {
    $i++;
  } elsif ($sf eq 'b') {
    $i--;
  }
  $i = $octave * 12 + $i;
# A4 = $Pitch, A4 = 12 * 4 + 9 = 57
  $freq = $Pitch * (2.0 ** (($i - 57) / 12.0));
  $freq = round($freq);
  return $freq;
}

sub round {
  my ($x) = @_;
  return ($x - int($x)) >= 0.5? int($x)+1 : int($x);
}

sub timeinmsec {
  my ($lengthSpec) = @_;
  my ($len);
  $len = notelength($lengthSpec);
  return $len * $DefaultUnitTime;
}

sub notelength {
  my ($lengthSpec) = @_;

  return ($lengthSpec eq '')? $DefaultLength: $lengthSpec;
}

sub volume {
  my ($volumeSpec) = @_;

  return ($volumeSpec eq '')? $DefaultVolume: $volumeSpec;
}

sub octave {
  my ($octaveSpec) = @_;
  my ($octave);

  if ($octaveSpec eq '') {
    $octave = $DefaultOctave;
  } elsif ($octaveSpec =~ /^+/) {
    $octave = $DefaultOctave + $';
  } elsif ($octaveSpec =~ /^-/) {
    $octave = $DefaultOctave - $';
  } else {
    $octave = $octaveSpec;
  }
  return $octave;
}

sub setDefaults {
  $Pitch = 440;
  $DefaultUnitTime = 1;
  $DefaultOctave = 3;
  $DefaultVolume = 64;
  $DefaultLength = 1;
}

